# -*- coding: utf-8 -*-
##code-section module-header #fill in your manual code here
import urllib
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions as cmfperms
##/code-section module-header

from zope import interface
from zope import component
from Products.ZCTextIndex import ParseTree
from Products.CMFPlone import utils
try:
    from Products.CMFPlone.utils import BrowserView
except:
    from Products.Five import BrowserView
from Products.azlinks.interfaces.azlinkfolderview import IAzLinkFolderView

class AzLinkFolderView(BrowserView):
    """
    """
    # zope3 interfaces
    interface.implements(IAzLinkFolderView)

    ##code-section class-header_AzLinkFolderView #fill in your manual code here
    ##/code-section class-header_AzLinkFolderView


    def __init__(self, *args, **kwargs):
        utils.BrowserView.__init__(self, *args, **kwargs)
        self._results = []
        self.startLetter = self.request.get('startLetter', None)
        self.category = self.request.get('category', None)
        self.searchText = self.request.get('searchText', None)
        if self.startLetter or self.category or self.searchText:
            self._findResults()
        context = utils.context(self)
        self.context_url = context.absolute_url()

    def azMatrix(self):
        """return a list of lists, so a row/column matrix.

        [['a', 'b', ... 'e'], ['f' ...], [... 'xy', 'z']]

        Each item will not be a simple string, but a dictionary with a
        title and a searchLink key.
        """

        # y left out intentionally:
        alphabet = 'abcdefghijklmnopqrstuvwxz'
        matrix = []
        for i in range(5):
            row = []
            for j in range(5):
                alphabetIndex = 5*i + j
                letter = alphabet[alphabetIndex]
                if letter == 'x':
                    # Here's where the 'y' comes back.
                    # 'xy' is a combo to get a nice 5x5 matrix.
                    letter = 'xy'
                entry = {}
                entry['title'] = letter
                entry['searchLink'] = generateLink(self.context_url,
                                                   'startLetter',
                                                   letter,
                                                   True)
                row.append(entry)
            matrix.append(row)
        return matrix

    def categories(self):
        """Return a list of categories.

        Each category will be a dictionary with a title and a
        searchLink key.
        """

        context = utils.context(self)
        cats = list(context.getCategories())
        # Add 'all'
        cats.append('All')
        cats.sort()
        results = []
        for cat in cats:
            item = {}
            item['title'] = cat
            item['searchLink'] = generateLink(self.context_url,
                                              'category',
                                              cat)
            results.append(item)
        return results

    def results(self):
        """Return results ordered by a..z.

        The results will be a list of dictionaries with keys 'letter'
        and 'lines'.

        Each result item will be a dictionary with a title and a link
        key.
        """

        context = utils.context(self)
        membership = getToolByName(context, 'portal_membership')

        # getting permission on folder since checking each link item
        # itself would make the processing of this search *much* slower
        can_edit = membership.checkPermission(cmfperms.ModifyPortalContent,
                                              context)
        letterCollectionMap = {}
        for item in self._results:
            firstLetter = item.getFirstLetter
            if not firstLetter:
                continue
            letterCollection = letterCollectionMap.get(
                firstLetter, [])
            # Grab the things we want from the item:
            dictItem = {}
            dictItem['title'] = item.Title
            dictItem['link'] = item.getRemoteUrl
            if can_edit:
                dictItem['edit_link'] = item.getURL() + '/edit'
            letterCollection.append(dictItem)
            letterCollectionMap[firstLetter] = letterCollection
        lettersFound = letterCollectionMap.keys()
        lettersFound.sort()
        output = []
        for letter in lettersFound:
            uppercaseLetter = letter.upper()
            lines = letterCollectionMap[letter]
            output.append({'letter': uppercaseLetter,
                           'lines': lines})
        return output

    def numberOfResults(self):
       """Return the number of results.

       If there are no results, just return 0.
       """

       return len(self._results)


    def _findResults(self):
        """Put all matching links into self._results.
        """

        context = utils.context(self)
        portal_catalog = getToolByName(context, 'portal_catalog')
        path = '/'.join(context.getPhysicalPath())
        contentType = 'AzLink'
        query = {'sort_on': 'sortable_title'}
        query['path'] = path
        query['portal_type'] = contentType
        if self.startLetter:
            query['getFirstLetter'] = self.startLetter
        if self.searchText:
            query['SearchableText'] = self.searchText
        try:
            results = portal_catalog.searchResults(query)
        except ParseTree.ParseError:
            results = []
        if results and self.category and self.category != 'All':
            results = [r for r in results
                       if self.category in r.getSelectedCategory]
        self._results = results

##code-section module-footer #fill in your manual code here

def generateLink(base_url, arg_name, chars, separate=False):
    """Generate a link reflecting the chars that are needed to be searched
    for and the given query portion as the argument name.

    Example usage::

      >>> generateLink('http://nohost/mybase', 'sl', 'a')
      'http://nohost/mybase?sl=a'
      
    Using multiple characters::
    
      >>> generateLink('http://nohost/mybase', 'sl', 'xy')
      'http://nohost/mybase?sl=xy'

    Using multiple characters with ``separate=True``::
    
      >>> generateLink('http://nohost/mybase', 'sl', 'xy', True)
      'http://nohost/mybase?sl=x&sl=y'

    """

    qs = ''
    if separate:
        for c in chars:
            qs += '%s=%s&' % (arg_name, urllib.quote(c))

        if qs:
            qs = qs[:-1]
    else:
        qs += '%s=%s' % (arg_name, urllib.quote(chars))

    if qs:
        qs = '?' + qs

    return base_url + qs

##/code-section module-footer


